/* gdt.c */

#include <gdt.h>

gdt_entry gdt[3];
gdt_ptr   gdtp;

/* set up a descriptor in the gdt */
void gdt_set_gate(int num, unsigned long base, unsigned long limit,
		  unsigned char access, unsigned char gran)
{
	/* setup descriptor base address */
	gdt[num].base_low    = (base & 0xFFFF);
	gdt[num].base_middle = (base >> 16) & 0xFF;
	gdt[num].base_high   = (base >> 24) & 0xFF;

	/* setup descriptor limits */
	gdt[num].limit_low   = (limit & 0xFFFF);
	gdt[num].granularity = ((limit >> 16) & 0x0F);

	/* set up granularity and access flags */
	gdt[num].granularity |= (gran & 0xF0);
	gdt[num].access      =  access;
	
} /* gdt_set_gate() */

/* initialize the gdt */
void gdt_install()
{
	/* set up gdt pointer and limit */
	gdtp.limit = (sizeof(gdt_entry) * 3) - 1;
	gdtp.base  = &gdt;

	/* null descriptor */
	gdt_set_gate(0, 0, 0, 0, 0);

	/* code segment descriptor */
	gdt_set_gate(1, 0, 0xFFFFFFFF, 0x9A, 0xCF);

	/* data segment descriptor */
	gdt_set_gate(2, 0, 0xFFFFFFFF, 0x92, 0xCF);

	/* flush old gdt and install new one */
	gdt_flush();

} /* gdt_install() */

/* end gdt.c */
