/* idt.c */

#include <idt.h>
#include <system.h>

void idt_set_gate(unsigned char num, unsigned long base,
		  unsigned short sel, unsigned char flags)
{
	/* base address of interrupt routine */
	idt[num].base_low  = (base & 0xFFFF);
	idt[num].base_high = (base >> 16) & 0xFFFF;

	/* segment used by idt entry, plus flags */
	idt[num].selector = sel;
	idt[num].zero     = 0;
	idt[num].flags    = flags;
} /* idt_set_gate() */

/* install idt */
void idt_install()
{
	/* set up idt pointer */
	idtp.limit = (sizeof(idt_entry) * 256) - 1;
	idtp.base  = &idt;

	/* clear idt by initializing to zeros */
	memset(&idt, 0, sizeof(idt_entry) * 256);

	/* finally, load the idt by calling the asm code in start.asm */
	idt_load();
} /* idt_install() */

/* end idt.c */
