/* screen.c */

#include <screen.h>
#include <system.h>
#include <string.h>

unsigned short *textmemptr;
int attrib = 0x0F;

/* initialize text cursor to coordinates (0,0) */
text_cursor cursor = { 0, 0 };

/* scroll screen */
void scroll(void)
{
	unsigned blank, temp;

	/* blank is defined as a space - give it a bg color too */
	blank = CHAR_BLANK | (attrib << 8);

	/* row 25 is last line, scroll up */
	if (cursor.y >= TEXT_SCREEN_MAX_Y)
	{
		/* move current text chunk back one line in the buffer */
		temp = cursor.y - TEXT_SCREEN_MAX_Y + 1;
		memcpy((unsigned char *)textmemptr, textmemptr + temp *
		       TEXT_SCREEN_MAX_X, (TEXT_SCREEN_MAX_Y - temp) *
		       TEXT_SCREEN_MAX_X * 2);

		/* set last line of text to blank */
		memsetw(textmemptr + (TEXT_SCREEN_MAX_Y - temp) *
			TEXT_SCREEN_MAX_X, blank, TEXT_SCREEN_MAX_X);
		cursor.y = TEXT_SCREEN_MAX_Y - 1;
	}
} /* scroll() */

/* update hardware cursor */
void move_cursor(void)
{
	unsigned temp;

	/* index of a linear chunk of memory */
	temp = cursor.y * TEXT_SCREEN_MAX_X + cursor.x;

	/* sends command to indicies 14 and 15 in crt control register of
	 * vga controller.  these are the high and low bytes of the index
	 * that show where the hardware cursor is to be blinky. */
	outportb(CRTC_INDEX, 14);
	outportb(CRTC_DATA , temp >> 8);
	outportb(CRTC_INDEX, 15);
	outportb(CRTC_DATA,  temp);
} /* move_cursor() */

/* clear screen */
void cls()
{
	unsigned blank;
	int i;

	/* blank is defined as a space - set bg color too */
	blank = CHAR_BLANK | (attrib << 8);

	/* sets entire screen to spaces */
	for (i = 0; i < TEXT_SCREEN_MAX_Y; i++)
		memsetw(textmemptr + i * TEXT_SCREEN_MAX_X, blank,
			TEXT_SCREEN_MAX_X);

	/* update virtual cursor, then hardware cursor */
	cursor.x = 0;
	cursor.y = 0;
	move_cursor();
} /* cls() */

/* set fore and back colors */
void set_text_color(unsigned char f, unsigned char b)
{
	/* top 4 bytes are bg, bottom 4 are fg */
	attrib = (b << 4) | (f & 0x0F);
} /* set_text_color() */

/* set text-mode vga pointer and clear screen */
void init_video(void)
{
	textmemptr = (unsigned short *)0xB8000;
	cls();
} /* init_video() */
	   
/* end screen.c */
