	;; start.asm

	[BITS 32]
	global start

start:
	mov	esp, _sys_stack	; points stack to new stack area
	jmp	stublet

	;; multiboot header must be 4 byte aligned so do that first
	ALIGN 4
mboot:
	;; set up multiboot macros for easier reading in actual header
	MB_PAGE_ALIGN	equ 1<<0
	MB_MEM_INFO	equ 1<<1
	MB_AOUT_KLUDGE	equ 1<<16
	MB_MAGIC	equ 0x1BADB002
	MB_FLAGS	equ MB_PAGE_ALIGN | MB_MEM_INFO | MB_AOUT_KLUDGE
	MB_CHKSUM	equ -(MB_MAGIC + MB_FLAGS)
	EXTERN		code, bss, end

	;; grub multiboot header - boot signature
	dd	MB_MAGIC
	dd	MB_FLAGS
	dd	MB_CHKSUM

	;; aout kludge - needs to be physical addresses - linker does this
	dd	mboot
	dd	code
	dd	bss
	dd	end
	dd	start

	;; we jmp here from start
stublet:
	extern	main		; main() is in main.c
	call	main		; jump to main
	jmp	$		; endless loop in case kernel returns

	;; global descriptor table
global gdt_flush		; allow c code to find gdt_flush()
extern gdtp			; gp is in gdt.h
gdt_flush:
	lgdt	[gdtp]		; load gdt
	mov	ax, 0x10	; offset in gdt to data segment
	mov	ds, ax
	mov	es, ax
	mov	fs, ax
	mov	gs, ax
	mov	ss, ax
	jmp	0x08:gdt_flush2	; offset to code segmend - far jmp
gdt_flush2:
	ret			; back to c code

	;; interrupt service routines
global idt_load
extern idtp
idt_load:
	lidt	[idtp]
	ret

	;; interrupt request (irq) routines

	;; stack - declare size of data before declaring identifier
	SECTION .bss
	resb 8192		; reserves 8kb
_sys_stack:	

	;; end start.asm